package com.hichina.main.back.hichinamainback.controller;

import com.hichina.main.back.hichinamainback.auth.AuthUserInfo;
import com.hichina.main.back.hichinamainback.auth.GoogleAuthenticator;
import com.hichina.main.back.hichinamainback.mapper.UserMapper;
import com.hichina.main.back.hichinamainback.model.User;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.mybatis.spring.boot.test.autoconfigure.AutoConfigureMybatis;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.data.mongo.AutoConfigureDataMongo;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.TestComponent;
import org.springframework.boot.test.context.TestConfiguration;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Bean;
import org.springframework.http.MediaType;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.web.servlet.MockMvc;

import java.util.List;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@ActiveProfiles("test")
@AutoConfigureMockMvc
@AutoConfigureDataMongo
@AutoConfigureMybatis
public class PublicLoginControllerTests {

    @MockBean
    private GoogleAuthenticator googleAuthenticator;

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private UserMapper userMapper;

    @Test
    public void should_auth_google_user() throws Exception {
        // given:
        Mockito.when(googleAuthenticator.verify(Mockito.anyString()))
                .thenReturn(new AuthUserInfo("xxx-yy", "test", "test@gmail.com", "https://example.com/pic.jpg"));

        // when
        mockMvc.perform(post("/api/public/login/google")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"accessToken\":\"fasfnen2e2rt\"}"))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.ok").value(true))
                .andExpect(jsonPath("$.data.email").value("test@gmail.com"));

        // then:
        List<User> userList = userMapper.findByEmail("test@gmail.com");
        Assertions.assertEquals(1, userList.size());

        User user = userList.get(0);
        Assertions.assertEquals("xxx-yy", user.getGoogleId());
        Assertions.assertEquals("test", user.getUsername());
        Assertions.assertEquals("https://example.com/pic.jpg", user.getProfileImageUrl());
        Assertions.assertEquals(-1, user.getPwdCode());
    }

    @Test
    public void should_not_auth_google_user_if_access_token_invalid() throws Exception {

        // given:
        Mockito.when(googleAuthenticator.verify(Mockito.anyString())).thenThrow(new BadCredentialsException(""));

        // when
        mockMvc.perform(post("/api/public/login/google")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"accessToken\":\"fasfnen2e2rt\"}"))
                .andExpect(status().isBadRequest())
                .andExpect(jsonPath("$.ok").value(true));
    }


}
